@@ -73,6 +73,8 @@ int (*_fcntl)(int fd, int cmd, ...);
 void *(*_mmap)(void *addr, size_t len, int prot, int flags, int fd, off_t offset);
 int (*_munmap)(void* addr, size_t len);
 
+FILE *(*_fopen)(const char *path, const char *mode);
+
 typedef struct ops {
 	int (*close)(int fd);
 	ssize_t (*write)(int fd, const void *buf, size_t n);
@@ -102,6 +104,26 @@ static int open_max;
 static int poll_fds_add = 0;
 static fd_t **fds;
 
+static int is_dsp_device(const char *pathname)
+{
+	if(!pathname) return 0;
+	if(strncmp(pathname,"/dev/dsp",8) == 0) return 1;
+	if(strncmp(pathname,"/dev/adsp",9) == 0) return 1;
+	if(strncmp(pathname,"/dev/audio",10) == 0) return 1;
+	if(strncmp(pathname,"/dev/sound/dsp",14) == 0) return 1;
+	if(strncmp(pathname,"/dev/sound/adsp",15) == 0) return 1;
+	if(strncmp(pathname,"/dev/sound/audio",16) == 0) return 1;
+	return 0;
+}
+
+static int is_mixer_device(const char *pathname)
+{
+	if(!pathname) return 0;
+	if(strncmp(pathname,"/dev/mixer",10) == 0) return 1;
+	if(strncmp(pathname,"/dev/sound/mixer",16) == 0) return 1;
+	return 0;
+}
+
 static int oss_pcm_fcntl(int fd, int cmd, ...)
 {
 	int result;
@@ -215,9 +237,7 @@ int open(const char *file, int oflag, ...)
 		mode = va_arg(args, mode_t);
 		va_end(args);
 	}
-	if (!strncmp(file, "/dev/dsp", 8) ||
-	    !strncmp(file, "/dev/adsp", 9) ||
-	    !strncmp(file, "/dev/audio", 10)) {
+	if (is_dsp_device(file)) {
 		fd = lib_oss_pcm_open(file, oflag);
 		if (fd >= 0) {
 			fds[fd] = calloc(sizeof(fd_t), 1);
@@ -230,7 +250,7 @@ int open(const char *file, int oflag, ...)
 			fds[fd]->oflags = oflag;
 			poll_fds_add += lib_oss_pcm_poll_fds(fd);
 		}
-	} else if (!strncmp(file, "/dev/mixer", 10)) {
+	} else if (is_mixer_device(file)) {
 		fd = lib_oss_mixer_open(file, oflag);
 		if (fd >= 0) {
 			fds[fd] = calloc(sizeof(fd_t), 1);
@@ -630,6 +650,16 @@ int select(int nfds, fd_set *rfds, fd_set *wfds, fd_set *efds,
 	return count1;
 }
 
+
+#include "stdioemu.c"
+
+FILE *fopen(const char* path, const char *mode) {
+	if(!is_dsp_device(path)) 
+		return _fopen (path, mode);
+	
+	return fake_fopen(path, mode);
+}
+
 #if 1
 # define strong_alias(name, aliasname) \
   extern __typeof (name) aliasname __attribute__ ((alias (#name)));
@@ -643,6 +673,7 @@ strong_alias(mmap, __mmap);
 strong_alias(munmap, __munmap);
 strong_alias(poll, __poll);
 strong_alias(select, __select);
+strong_alias(fopen, __fopen);
 #else
 int dup(int fd)
 {
@@ -712,4 +743,5 @@ static void initialize()
 	_munmap = dlsym(RTLD_NEXT, "munmap");
 	_select = dlsym(RTLD_NEXT, "select");
 	_poll = dlsym(RTLD_NEXT, "poll");
+	_fopen = dlsym(RTLD_NEXT, "fopen");
 }
